using UnityEngine;
using System.Collections;
using System.Collections.Generic;


public class AdaptiveTask : Decorator 
{
	
	List<Observation> observations = new List<Observation>();
	List<HypotheseNode> hypotheses = new List<HypotheseNode>(); 
	
	
	public AdaptiveTask(List<Observation> observations, List<HypotheseNode> Hypothese)
	{
		this.observations = observations; 
		this.hypotheses = Hypothese; 
		
	}
	
	public override BehaResult run (MonoBehaviour agent)
	{
		
		foreach (Observation o in observations)
		{
			//This is to update the observation
			o.check();
		}
		
		
		//Check probability = 1
		float pTotal=0.0f; 
		
		foreach(HypotheseNode h in hypotheses)
		{
			pTotal += h.Prob; 	
		}
		
		if(pTotal>1.01|pTotal<0.99)
			return BehaResult.NodeProblem;
		
		
		//Calculate Probdistribution
		List<HypotheseNode> hypNotToSetZero= new List<HypotheseNode>();
		foreach (Observation o in observations)
		{
			//This is to update the observation
			if (o.observed)
			{
 
				//Get what is under the bayesian formula
				float undersum = 0; 
				foreach(HypotheseLinkObservation hUnder in o.hyps)
				{
					undersum += hUnder.prob*hUnder.hyp.prob;
				}
				
				foreach(HypotheseLinkObservation h in o.hyps)
				{
					//Get what is under the bayesian formula
					h.hyp.prob = h.prob*h.hyp.prob/undersum;
					hypNotToSetZero.Add(h.hyp);
				}
				foreach(HypotheseNode hypAll in hypotheses)
				{
					if(!hypNotToSetZero.Contains(hypAll))
				   	{
						hypAll.Prob = 0.0f; 
					}
				}
					
			o.observed = false; 
			}
		}

				
		
		//Get an action according to probability
		float rand = Random.Range(0,101)/100f; 
		float tempR = 0;
		
		
		
		foreach(HypotheseNode h in hypotheses)
		{
			tempR += h.Prob;
			
			if(rand < tempR)
			{
				child = h;
				break; 
			}
		}
		
		
		return base.run(agent);
	}

}
